@@ -245,33 +245,35 @@ is inserted before the cursor, the short name is expanded to its address."
 
 (defun mew-circular-complete-case ()
   (interactive)
-  (let (cases oldcase newcase insert-:)
-    (save-excursion
-      (if (search-backward "," nil t)
-	  (forward-char 1)
-	(beginning-of-line))
-      (if (looking-at mew-regex-case2)
-	  (progn
-	    (setq oldcase (mew-match-string 1))
-	    (delete-region (match-beginning 1) (match-end 1)))
-	(setq oldcase mew-case-default)
-	(setq insert-: t))
-      (if (setq cases (member oldcase mew-config-cases))
-	  (if (> (length cases) 1)
-	      (setq newcase (nth 1 cases))
-	    (setq newcase (car mew-config-cases)))
-	(setq newcase mew-case-default))
-      (if (string= newcase mew-case-default)
-	  (unless insert-: (delete-char 1))
-	(insert newcase)
-	(if insert-: (insert ":"))))
-    (if (or (= (point) (mew-minibuf-point-min))
-	    (save-excursion
+  (if mew-input-folder-search-direction
+      (mew-input-folder-self-insert)
+    (let (cases oldcase newcase insert-:)
+      (save-excursion
+	(if (search-backward "," nil t)
+	    (forward-char 1)
+	  (beginning-of-line))
+	(if (looking-at mew-regex-case2)
+	    (progn
+	      (setq oldcase (mew-match-string 1))
+	      (delete-region (match-beginning 1) (match-end 1)))
+	  (setq oldcase mew-case-default)
+	  (setq insert-: t))
+	(if (setq cases (member oldcase mew-config-cases))
+	    (if (> (length cases) 1)
+		(setq newcase (nth 1 cases))
+	      (setq newcase (car mew-config-cases)))
+	  (setq newcase mew-case-default))
+	(if (string= newcase mew-case-default)
+	    (unless insert-: (delete-char 1))
+	  (insert newcase)
+	  (if insert-: (insert ":"))))
+      (if (or (= (point) (mew-minibuf-point-min))
+	      (save-excursion
+		(forward-char -1)
+		(looking-at "[:,]")))
+	  (if (search-forward "," nil t)
 	      (forward-char -1)
-	      (looking-at "[:,]")))
-	(if (search-forward "," nil t)
-	    (forward-char -1)
-	  (goto-char (point-max))))))
+	    (goto-char (point-max)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
