@@ -401,7 +401,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	    (mew-header-arrange (point-min) (point-max)))
 	  (setq start (point))
 	  (save-excursion
-	    (insert-buffer-substring buf beg end)
+	    (mew-insert-buffer-substring buf beg end)
 	    (mew-cs-decode-region start (point-max) tocs)
 	    (mew-highlight-body-region start (point-max)))
 	  ;; Page breaks
