@@ -119,8 +119,7 @@
 	    (if (> (mew-header-count-field (car fields)) 1)
 		(mew-encode-error (format "Multiple %s!" (car fields))))
 	    (setq fields (cdr fields))))
-	(mapcar (function mew-header-combine-field)
-		mew-multiple-field-combine-list)))))
+	(mapcar 'mew-header-combine-field mew-multiple-field-combine-list)))))
 
 (defun mew-encode-ask-subject ()
   (when (and mew-ask-subject (not (mew-header-existp mew-subj:)))
@@ -140,18 +139,11 @@
 	(when (and mew-ask-fcc
 		   (not (file-directory-p (mew-expand-folder folder))))
 	  (if (y-or-n-p (format "%s does not exist. Create it? " folder))
-	      (mew-folder-check folder 'force-to-create)
+	      (mew-local-folder-check folder)
 	    (error "Edit Fcc:")))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
 
-;; xxx NetNews: ""?
-(defun mew-encode-ask-newsgroups ()
-  (when (and mew-ask-newsgroups (mew-header-existp mew-newsgroups:))
-    (unless (y-or-n-p "Do you want to post to NetNews? ")
-      (save-excursion
-	(mew-header-delete-lines (list mew-newsgroups:))))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; To:, Cc:
@@ -167,7 +159,7 @@
 	(narrow-to-region (point-min) (mew-header-end))
 	(goto-char (point-min))
 	(while (re-search-forward regex nil t)
-	  (setq key (mew-match 1))
+	  (setq key (match-string 1))
 	  (setq start (match-end 0))
 	  (forward-line)
 	  (while (looking-at mew-lwsp)
@@ -177,8 +169,7 @@
 	  (delete-region start (1- (point)))
 	  (backward-char 1)
 	  ;;
-	  (setq vals (mapcar (function mew-chop)
-			     (mew-split-quoted val ?, ?: ?\;)))
+	  (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\;)))
 	  (while vals
 	    (setq val (car vals))
 	    (setq vals (cdr vals))
@@ -187,27 +178,27 @@
 	     ((string-match "[^:]+:[^;]+;" val)
 	      (let ((mew-alias-expand-prefix nil))
 		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
-		(setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+		(setq addrs (mapcar 'mew-addrstr-append-domain addrs))
 		(setq ins (list (concat mew-alias-expand-prefix ":;")))))
 	     ((and (setq addr (mew-addrstr-parse-address val))
 		   (string-match (concat "\\(.*<\\)" (regexp-quote addr) "\\(>.*\\)")
 				 val))
-	      (setq prefix (mew-match 1 val))
-	      (setq suffix (mew-match 2 val))
+	      (setq prefix (match-string 1 val))
+	      (setq suffix (match-string 2 val))
 	      (setq addr (mew-addrstr-append-domain addr))
 	      (setq addrs (list addr))
 	      (setq ins (list (concat prefix addr suffix))))
 	     (t
 	      (let ((mew-alias-expand-prefix nil))
 		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
-		(setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+		(setq addrs (mapcar 'mew-addrstr-append-domain addrs))
 		(if mew-alias-expand-prefix
 		    (setq ins (list (concat mew-alias-expand-prefix ":;")))
 		  (setq ins (copy-sequence addrs))))))
 	    (setq insl (nconc insl ins))
 	    (unless (string-match "bcc" key) ;; Removing Bcc:
 	      (setq ret (nconc ret addrs))))
-	  (insert " " (mapconcat (function identity) insl ", "))
+	  (insert " " (mapconcat 'identity insl ", "))
 	  (setq insl nil)
 	  (forward-line))))
     ret))
@@ -240,8 +231,6 @@
 (defun mew-encode-id-date (pnm case resentp)
   (let ((msgid (mew-smtp-message-id case))
 	(time (current-time)))
-    (mew-smtp-set-msgid pnm msgid)
-    (mew-smtp-set-logtime pnm (mew-time-ctz-to-logtime time))
     (cond
      (resentp
       (mew-header-delete-lines (list mew-resent-date: mew-resent-message-id:))
@@ -254,45 +243,28 @@
       (save-excursion
 	(goto-char (point-min))
 	(mew-header-insert mew-date: (mew-time-ctz-to-rfc time))
-	(mew-header-insert mew-message-id: msgid))))))
+	(mew-header-insert mew-message-id: msgid))))
+    (list msgid (mew-time-ctz-to-logtime time))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; From:, Sender:
 ;;;
 
-(defun mew-encode-from-sender (pnm case resentp)
+(defun mew-encode-check-sender (resentp)
   (if (< 1 (length (mew-addrstr-parse-address-list
 		    (mew-header-get-value
 		     (if resentp mew-resent-sender: mew-sender:)))))
-      (mew-encode-error "Sender: must contain one address!"))
-  (let* ((mail-from (mew-addrstr-parse-address (mew-smtp-mail-from case)))
-	 (from (mew-addrstr-parse-address-list
-		(mew-header-get-value
-		 (if resentp mew-resent-from: mew-from:))))
-	 (nfrom (length from))
-	 insert-from)
-    (cond
-     ((= nfrom 0)
-      (setq insert-from (mew-from case))
-      (setq from (mew-addrstr-parse-address insert-from))
-      (if (null mail-from) (setq mail-from from)))
-     ((= nfrom 1)
-      (setq from (car from))
-      (if (null mail-from) (setq mail-from from)))
-     (t
-      (if (null mail-from)
-	  ;; Which address is suitable form MAIL FROM?
-	  (setq mail-from (mew-addrstr-parse-address (mew-from case))))))
-    ;; MUST be an address only (excluding phrase)
-    (mew-smtp-set-from pnm mail-from)
-    ;;
-    (when insert-from
+      (mew-encode-error "Sender: must contain one address!")))
+
+(defun mew-encode-from (case resentp)
+  (unless (mew-header-existp mew-from:)
+    (let ((from (mew-from case)))
       (save-excursion
 	(goto-char (point-min))
 	(if resentp
-	    (mew-header-insert mew-resent-from: insert-from)
-	  (mew-header-insert mew-from: insert-from))))))
+	    (mew-header-insert mew-resent-from: from)
+	  (mew-header-insert mew-from: from))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -304,7 +276,7 @@
   (if (and mew-use-auto-alias mew-addrbook-append-domain-p)
       ;; If mew-addrbook-append-domain-p is nil, automatic
       ;; short names would be conflicted to local users.
-      (mapcar (function mew-addrbook-alias-add)
+      (mapcar 'mew-addrbook-alias-add
 	      (mew-header-parse-address-list
 	       (if resentp
 		   (list mew-resent-to: mew-resent-cc:)
@@ -366,9 +338,7 @@
    mew-cs-dummy mew-cs-text-for-write
    (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)
    (clear-visited-file-modtime)
-   (if (and buffer-auto-save-file-name
-	    (file-exists-p buffer-auto-save-file-name))
-       (delete-file buffer-auto-save-file-name))
+   (mew-delete-file buffer-auto-save-file-name)
    (set-buffer-modified-p nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -376,29 +346,29 @@
 ;;; Encoding a message
 ;;;
 
-(defun mew-encode (pnm case resentp &optional privacy signer headerp)
+(defun mew-smtp-encode (pnm case resentp fcc &optional privacy signer headerp)
   (let ((ret t))
     (if (mew-debug 'encode)
 	(let ((debug-on-error t))
-	  (mew-encode-message pnm case resentp privacy signer headerp))
+	  (mew-smtp-encode-message pnm case resentp fcc privacy signer headerp))
       (condition-case nil
-	  (mew-encode-message pnm case resentp privacy signer headerp)
+	  (mew-smtp-encode-message pnm case resentp fcc privacy signer headerp)
 	(error
 	 (mew-encode-undo)
 	 (message (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
-(defun mew-encode-message (pnm case resentp &optional privacy signer headerp)
+(defun mew-smtp-encode-message (pnm case resentp fcc &optional privacy signer headerp)
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
   (run-hooks 'mew-make-message-hook)
-  (let (multip recipients)
-    (mew-smtp-set-raw-header
-     pnm (mew-buffer-substring (point-min) (mew-header-end)))
+  (let (multip recipients msgid-logtime)
     (goto-char (point-min))
     (mew-encode-remove-illegal-fields)
+    (mew-smtp-set-raw-header
+     pnm (mew-buffer-substring (point-min) (mew-header-end)))
     ;; Destination check
     (setq recipients (mew-encode-canonicalize-address resentp))
     ;; Bcc: is not included.
@@ -427,15 +397,15 @@
     (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
     (mew-smtp-set-dcc pnm (mew-encode-delete-dcc resentp))
     (mew-smtp-set-bcc pnm (mew-encode-delete-bcc resentp))
-    (mew-encode-id-date pnm case resentp)
-    (mew-encode-from-sender pnm case resentp)
+    (mew-smtp-set-fcc pnm fcc)
+    ;;
+    (mew-encode-check-sender resentp)
+    (mew-encode-from case resentp)
     (mew-encode-learn-aliases resentp)
     ;;
-    (mew-smtp-set-user pnm (mew-smtp-user case))
-    (mew-smtp-set-auth pnm (mew-smtp-auth case))
-    (mew-smtp-set-auth-list pnm (mew-smtp-auth-list case))
-    (mew-smtp-set-helo-domain pnm (mew-smtp-helo-domain case))
-    (mew-smtp-set-queue pnm (mew-queue-folder case))
+    (setq msgid-logtime (mew-encode-id-date pnm case resentp))
+    (mew-smtp-set-msgid pnm (nth 0 msgid-logtime))
+    (mew-smtp-set-logtime pnm (nth 1 msgid-logtime))
     ;;
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
@@ -453,6 +423,77 @@
     (mew-overlay-delete-buffer)
     (message "Making a message...done")))
 
+(defun mew-nntp2-encode (pnm case fcc &optional privacy signer headerp)
+  (let ((ret t))
+    (if (mew-debug 'encode)
+	(let ((debug-on-error t))
+	  (mew-nntp2-encode-message pnm case fcc privacy signer headerp))
+      (condition-case nil
+	  (mew-nntp2-encode-message pnm case fcc privacy signer headerp)
+	(error
+	 (mew-encode-undo)
+	 (message (mew-tinfo-get-encode-err))
+	 (setq ret nil))))
+    ret))
+
+(defun mew-nntp2-encode-message (pnm case fcc &optional privacy signer headerp)
+  (mew-set-buffer-multibyte t)
+  (if (buffer-modified-p) (save-buffer)) ;; to make backup
+  (widen)
+  (run-hooks 'mew-make-message-hook)
+  (let (multip newsgroups msgid-logtime)
+    (goto-char (point-min))
+    (mew-encode-remove-illegal-fields)
+    (mew-nntp2-set-raw-header
+     pnm (mew-buffer-substring (point-min) (mew-header-end)))
+    ;;
+    (setq newsgroups (mew-header-get-value mew-newsgroups:))
+    (mew-nntp2-set-newsgroups pnm newsgroups)
+    ;; Let's backup
+    (unless headerp
+      (if (mew-attach-p)
+	  (progn
+	    (mew-attach-clear)
+	    (cond
+	     ((mew-encode-syntax-have-multiple-parts)
+	      (setq multip t))
+	     ((mew-encode-syntax-have-one-part)
+	      ;; A user may want to do any MIME stuff to the body
+	      (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax)))
+	     (t
+	      ;; xxx error but ...
+	      )))
+	(setq mew-encode-syntax
+	      (mew-encode-syntax-single "text.txt" (list mew-ct-txt))))
+      (mew-encode-make-backup))
+    ;; Errors can be caused from here.
+    (unless newsgroups (mew-encode-error "No newsgroups!"))
+    ;; Header modifications which are not remained.
+    (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
+    (mew-nntp2-set-fcc pnm fcc)
+    ;;
+    (mew-encode-from case nil)
+    ;;
+    (setq msgid-logtime (mew-encode-id-date pnm case nil))
+    (mew-nntp2-set-msgid pnm (nth 0 msgid-logtime))
+    (mew-nntp2-set-logtime pnm (nth 1 msgid-logtime))
+    ;;
+    (goto-char (mew-header-end))
+    (forward-line) ;; necessary for PGP
+    ;;
+    (message "Making a message...")
+    ;; save syntax before setting privacy
+    (unless headerp
+      (mew-encode-set-privacy pnm privacy)
+      (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
+	(if multip
+	    (mew-encode-make-multi)
+	  (mew-encode-make-single))))
+    (mew-encode-make-header nil)
+    (mew-encode-save-draft)
+    (mew-overlay-delete-buffer)
+    (message "Making a message...done")))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
 ;;;
 ;;; Encoding singlepart
@@ -573,7 +614,7 @@
 	 (insert-file-contents file)))
       (when linebasep
 	(goto-char beg)
-	(while (search-forward "\n" nil t) (replace-match "\r\n" nil t)))
+	(mew-lf-to-crlf))
       (base64-encode-region beg (point-max))
       (goto-char (point-max))
       (insert "\n"))
@@ -590,10 +631,8 @@
 	  (delete-region beg (point-max)))
 	(mew-piolet
 	 mew-cs-text-for-read mew-cs-dummy
-	 (apply (function call-process) mew-prog-mime-encode
-		file t nil opt))
-	(if (and textp (file-exists-p file))
-	    (delete-file file))))
+	 (apply 'call-process mew-prog-mime-encode file t nil opt))
+	(if textp (mew-delete-file file))))
      (t
       (mew-encode-error (concat mew-prog-mime-encode " does not exist"))))
     (list (if charset (list "charset" charset)) cte)))
@@ -710,7 +749,7 @@
     (setq cte-pos (point))
     (and cd (mew-header-insert mew-cd: cd))
     (while (< cnt len)
-      (insert (concat "\n--" boundary "\n"))
+      (insert "\n--" boundary "\n")
       (if (mew-syntax-multipart-p (aref syntax cnt))
 	  (setq 8bitp (mew-encode-multipart 
 		       (aref syntax cnt) fullname (1+ depth)))
@@ -719,7 +758,7 @@
 		     (if (eq cnt mew-syntax-magic) buffered nil))))
       (if 8bitp (setq 8bit-cnt (1+ 8bit-cnt)))
       (setq cnt (1+ cnt)))
-    (insert (concat "\n--" boundary "--\n"))
+    (insert "\n--" boundary "--\n")
     (save-excursion
       (goto-char cte-pos)
       (mew-header-insert mew-cte: (if (> 8bit-cnt 0) mew-8bit mew-7bit)))
@@ -766,7 +805,7 @@
 	  (setq sbeg (point-min) send (point-max))))
     (unless mew-cs-text-for-net
       (goto-char sbeg) ;; just in case
-      (while (search-forward "\n" nil t) (replace-match "\r\n" nil t))
+      (mew-lf-to-crlf)
       (setq send (point-max)))
     (setq file (mew-make-temp-name))
     (mew-frwlet
@@ -793,7 +832,7 @@
     (condition-case nil
 	(setq fc (funcall func file1 decrypters))
       (error
-       (if (file-exists-p file1) (delete-file file1))
+       (mew-delete-file file1)
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mle mew-temp-dir))))
@@ -801,9 +840,9 @@
     (setq errmsg (nth 4 fc))
     (if errmsg
 	(progn
-	  (if (file-exists-p file1) (delete-file file1))
-	  (if (file-exists-p file2) (delete-file file2))
-	  (if (file-exists-p file3) (delete-file file3))
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
+	  (mew-delete-file file3)
 	  (mew-tinfo-set-privacy-err t)
 	  (mew-tinfo-set-privacy-type nil)
 	  (mew-draft-mode-name)
@@ -823,9 +862,9 @@
        (mew-encode-syntax-single file3 mew-type-apo cte3))
       (insert (format "\n--%s--\n" boundary))
       ;; Throw away the garbage 
-      (if (file-exists-p file1) (delete-file file1))
-      (if (file-exists-p file2) (delete-file file2))
-      (if (file-exists-p file3) (delete-file file3)))))
+      (mew-delete-file file1)
+      (mew-delete-file file2)
+      (mew-delete-file file3))))
 
 (defun mew-encode-multipart-signed (ct proto depth)
   ;; called in the narrowed region
@@ -839,7 +878,7 @@
     (condition-case nil
 	(setq fmc (funcall func file1))
       (error
-       (if (file-exists-p file1) (delete-file file1))
+       (mew-delete-file file1)
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mls mew-temp-dir))))
@@ -848,8 +887,8 @@
     (setq ct2 (nth 4 fmc) cdp2 (nth 5 fmc))
     (if errmsg
 	(progn
-	  (if (file-exists-p file1) (delete-file file1))
-	  (if (file-exists-p file2) (delete-file file2))
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
 	  (mew-tinfo-set-privacy-err t)
 	  (mew-tinfo-set-privacy-type nil)
 	  (mew-draft-mode-name)
@@ -870,8 +909,8 @@
        (mew-encode-syntax-single file2 (or ct2 (list proto)) cte2 nil nil cdp2))
       (insert (format "\n--%s--\n" boundary))
       ;; Throw away the garbage 
-      (if (file-exists-p file1) (delete-file file1))
-      (if (file-exists-p file2) (delete-file file2)))))
+      (mew-delete-file file1)
+      (mew-delete-file file2))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -889,8 +928,7 @@
        ((and (mew-case-equal cte mew-b64) (fboundp 'base64-encode-region))
 	(when linebasep
 	  (goto-char (point-min))
-	  (while (search-forward "\n" nil t)
-	    (replace-match "\r\n" nil t)))
+	  (mew-lf-to-crlf))
 	(base64-encode-region (point-min) (point-max))
 	(goto-char (point-max))
 	(insert "\n"))
@@ -907,9 +945,8 @@
 	   (delete-region (point-min) (point-max)))
 	  (mew-piolet
 	   mew-cs-text-for-read mew-cs-dummy
-	   (apply (function call-process) mew-prog-mime-encode
-		  file t nil opt))
-	  (if (file-exists-p file) (delete-file file))))
+	   (apply 'call-process mew-prog-mime-encode file t nil opt))
+	  (mew-delete-file file)))
        (t
 	(mew-encode-error (concat mew-prog-mime-encode " does not exist")))))))
 
@@ -1035,7 +1072,7 @@
       (while t
 	(unless (re-search-forward bregex nil t)
 	  (mew-encode-error (format "No last boundary for %s" ct)))
-	(setq break (string= (regexp-quote (mew-match 0)) ebound))
+	(setq break (string= (regexp-quote (match-string 0)) ebound))
 	(forward-line) ;; the beginning of the next part
 	(save-excursion
 	  (forward-line -1)
