@@ -453,7 +453,7 @@ also ignored."
 
 (defun mew-folder-case (folder)
   ;; "case" must be distinguished from drive
-  (if (string-match "^\\([-a-zA-Z0-9_][-a-zA-Z0-9_]+\\):" folder)
+  (if (string-match mew-regex-case folder)
       (cons (match-string 1 folder) (substring folder (match-end 0)))))
 
 (defun mew-folder-folder (folder)
@@ -1093,6 +1093,9 @@ Does not examine containing directories for links."
       (aset vec i nil)
       (setq i (1+ i)))))
 
+(defsubst mew-net-get-ssh-process (pnm)
+  (aref (mew-info pnm) 3))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Buffer local info
