@@ -488,22 +488,32 @@
   (define-key mew-input-map "\t"     'mew-input-complete)
   (define-key mew-input-map [tab]    'mew-input-complete)
   (define-key mew-input-map "?"      'mew-input-complete)
+  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-case)
   (define-key mew-input-map ","      'mew-input-comma))
 
 (unless mew-input-folder-map
   (setq mew-input-folder-map (copy-keymap mew-input-map))
-  (define-key mew-input-folder-map [t]      'mew-input-folder-self-insert)
+  (let ((c 33)) ;; excluding SPC
+    (while (<= c 128) ;; including delete
+      (define-key mew-input-folder-map (char-to-string c)
+	'mew-input-folder-self-insert)
+      (setq c (1+ c))))
   (define-key mew-input-folder-map "\C-g"   'mew-abort-minibuffer2)
   (define-key mew-input-folder-map "\r"     'mew-exit-minibuffer2)
   (define-key mew-input-folder-map "\n"     'mew-exit-minibuffer2)
   (define-key mew-input-folder-map [return] 'mew-exit-minibuffer2)
+  (define-key mew-input-folder-map [delete] 'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map [backspace] 'mew-input-folder-self-insert)
   (define-key mew-input-folder-map "+"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "%"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "$"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "-"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "*"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map ","      'mew-input-folder-comma)
   (define-key mew-input-folder-map "\C-s"   'mew-input-folder-search-forward)
-  (define-key mew-input-folder-map "\C-r"   'mew-input-folder-search-backward))
+  (define-key mew-input-folder-map "\C-r"   'mew-input-folder-search-backward)
+  (define-key mew-input-folder-map "\C-n"   'mew-input-folder-search-forward)
+  (define-key mew-input-folder-map "\C-p"   'mew-input-folder-search-backward))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
