@@ -39,9 +39,7 @@
   (when (or (not (and mew-input-comma-function
 		      (fboundp mew-input-comma-function)))
 	    (funcall mew-input-comma-function))
-    (if mew-xemacs-p
-	(insert (event-to-character last-command-event))
-      (insert last-command-event))))
+    (insert ",")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -67,6 +65,8 @@ at completions."
 (defvar mew-input-folder-search-key nil)
 (defvar mew-input-folder-search-match nil)
 (defvar mew-input-folder-search-original nil)
+(defvar mew-input-folder-search-point nil)
+(defvar mew-input-folder-search-multi nil)
 
 (defun mew-input-folder-prefix ()
   (interactive)
@@ -101,21 +101,30 @@ at completions."
 
 (defun mew-input-folder-self-insert ()
   (interactive)
-  (let* ((str-vec (this-command-keys))
+  (let* ((key (if mew-xemacs-p
+		  (event-key last-command-event)
+		(this-command-keys)))
 	 last-str gfunc)
     (cond
-     ((stringp str-vec)
-      (setq last-str str-vec)
-      (setq gfunc (lookup-key (current-global-map) str-vec)))
-     ((vectorp str-vec)
-      (setq gfunc (lookup-key (current-global-map) str-vec))
+     ((mew-characterp key) ;; XEmacs
+      (setq key (char-to-string key))
+      (setq last-str key)
+      (setq gfunc (lookup-key (current-global-map) key)))
+     ((stringp key)
+      (setq last-str key)
+      (setq gfunc (lookup-key (current-global-map) key)))
+     ((symbolp key) ;; XEmacs
+      (setq gfunc (lookup-key (current-global-map) key)))
+     ((vectorp key)
+      (setq gfunc (lookup-key (current-global-map) key))
       (unless gfunc
-	(setq str-vec (lookup-key function-key-map str-vec))
-	(if (vectorp str-vec)
-	    (setq gfunc (lookup-key (current-global-map) str-vec))))))
+	(setq key (lookup-key function-key-map key))
+	(if (vectorp key)
+	    (setq gfunc (lookup-key (current-global-map) key))))))
     (if mew-input-folder-search-direction
 	(cond
-	 ((or (equal str-vec "\177") (equal str-vec [127]))
+	 ((or (equal key "\177") (equal key [127])
+	      (equal key 'delete) (equal key 'backspace))
 	  (if (null mew-input-folder-search-key)
 	      (mew-input-folder-display "not allowed")
 	    (setq mew-input-folder-search-key
@@ -162,7 +171,7 @@ at completions."
 	  (progn
 	    (goto-char (match-beginning 0))
 	    (setq mew-input-folder-search-match
-		  (buffer-substring
+		  (mew-buffer-substring
 		   (save-excursion (beginning-of-line) (point))
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
@@ -185,7 +194,7 @@ at completions."
 	  (progn
 	    (goto-char (match-end 0))
 	    (setq mew-input-folder-search-match
-		  (buffer-substring
+		  (mew-buffer-substring
 		   (save-excursion (beginning-of-line) (point))
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
@@ -199,9 +208,6 @@ at completions."
 
 (defun mew-input-folder-search-forward ()
   (interactive)
-  (unless mew-input-folder-search-direction
-    (setq mew-input-folder-search-original
-	  (buffer-substring (mew-minibuf-point-min) (point-max))))
   (cond
    ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
     (mew-input-folder-display "no match"))
@@ -216,9 +222,6 @@ at completions."
 
 (defun mew-input-folder-search-backward ()
   (interactive)
-  (unless mew-input-folder-search-direction
-    (setq mew-input-folder-search-original
-	  (buffer-substring (mew-minibuf-point-min) (point-max))))
   (cond
    ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
     (mew-input-folder-display "no match"))
@@ -232,10 +235,19 @@ at completions."
     (mew-input-folder-search-setup))))
 
 (defun mew-input-folder-search-setup (&optional min)
-  (let* ((case:fld (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
-	 (case (car (mew-folder-case case:fld)))
-	 (fld (mew-folder-folder case:fld))
-	 alist buf)
+  (let* (case:fld case fld alist buf)
+    (save-excursion
+      (goto-char (point-max))
+      (if (search-backward "," nil t)
+	  (setq mew-input-folder-search-point
+		(- (match-end 0) (mew-minibuf-point-min)))))
+    (setq mew-input-folder-search-original
+	  (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+    (setq case:fld mew-input-folder-search-original)
+    (if mew-input-folder-search-point
+	(setq case:fld (substring case:fld mew-input-folder-search-point)))
+    (setq case (car (mew-folder-case case:fld)))
+    (setq fld (mew-folder-folder case:fld))
     (mew-input-folder-display)
     (if (or (null fld) (string= fld ""))
 	(setq fld (mew-proto case)))
@@ -264,6 +276,9 @@ at completions."
 	(delete-region (mew-minibuf-point-min) (point-max))
 	(cond
 	 (mew-input-folder-search-match
+	  (when mew-input-folder-search-point
+	    (insert (substring mew-input-folder-search-original
+			       0 mew-input-folder-search-point)))
 	  (insert mew-input-folder-search-match))
 	 (mew-input-folder-search-original
 	  (insert mew-input-folder-search-original)))
@@ -280,11 +295,18 @@ at completions."
 	(mew-input-folder-clean-up))
     (abort-recursive-edit)))
 
+(defun mew-input-folder-comma ()
+  (interactive)
+  (if (and (null mew-input-folder-search-direction)
+	   mew-input-folder-search-multi)
+      (insert ",")))
+
 (defun mew-input-folder-clean-up ()
   (setq mew-input-folder-search-direction nil)
   (setq mew-input-folder-search-key nil)
   (setq mew-input-folder-search-match nil)
-  (setq mew-input-folder-search-original nil))
+  (setq mew-input-folder-search-original nil)
+  (setq mew-input-folder-search-point nil))
 
 (defun mew-input-folder (default &optional local-only)
   "Input a folder from the minibuffer.
@@ -295,6 +317,7 @@ This function returns a folder."
 	folder)
     (mew-input-clear)
     (mew-input-folder-clean-up)
+    (setq mew-input-folder-search-multi nil)
     (if local-only
 	(setq mew-input-complete-function 'mew-complete-local-folder)
       (setq mew-input-complete-function 'mew-complete-folder))
@@ -320,6 +343,7 @@ This function returns a folder."
       (setq init (mew-join "," default))))
     (mew-input-clear)
     (mew-input-folder-clean-up)
+    (setq mew-input-folder-search-multi t)
     (if proto
 	(if (mew-folder-imapp proto)
 	    (setq mew-input-complete-function 'mew-complete-imap-folder)
