@@ -142,7 +142,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   '("DUMMY"
     "^WARNING:[ -9;-~\n]+belongs? to:"
     "^WARNING:[ -9;-~\n]+belongs? to:"
-    "There is no indication that this key really belongs to the owner")) ;;xxx ;;koie
+    "There is no indication that this key really belongs to the owner"))
 
 (defconst mew-pgp-msg-no-vrfkey
   '("Key matching" "unknown keyid" "key does not meet" "public key not found"))
