@@ -152,37 +152,46 @@ mew-refile-guess-from-me-is-special."
   ;; Guess folders by user-configured alist.
   (mew-refile-guess-by-alist1 mew-refile-guess-alist))
 
-(defun mew-refile-guess-by-alist1 (alist)
-  (let (name header sublist key val ent ret)
-    (while alist
-      (setq name (car (car alist)))
-      (setq sublist (cdr (car alist)))
+(defun mew-refile-guess-by-alist1 (rule)
+  "Return a guessed folder according to the RULE. The syntax of
+RULE is as follows:
+    rule      ::= '<rule>
+    <rule>    ::= ((<key> <alist>) (<key> <alist>) ... [<special>])
+    <alist>   ::= (<value> . <folder>|<rule>) (<value> . <folder>|<rule>) ...
+    <special> ::= (t . <folder>) | (nil . <folder>)
+
+There are two special <key>s: @samp{nil} and @samp{t}. @samp{nil} is
+used to specify <folder> to be returned when nothing is
+guessed. @samp{t} can specify <folder> to be returned in addition to
+guessed values. "
+  (let (key alist val f-or-r header ent ret)
+    (while rule
+      (setq key (car (car rule)))
+      (setq alist (cdr (car rule)))
       (cond
-       ((eq name t)
-	(mew-addq ret sublist))
-       ((eq name nil)
-	(or ret (mew-addq ret sublist)))
-       (t
-	(setq header (mew-header-get-value name))
-	(when header
-	  (while sublist
-	    (setq key (car (car sublist)))
-	    (setq val (cdr (car sublist)))
-	    (setq ent nil)
-	    (when (and (stringp key) (string-match key header))
-	      (cond
-	       ((stringp val)
-		(setq ent (mew-refile-guess-by-alist2 key header val)))
-	       ((listp val)
-		(setq ent (mew-refile-guess-by-alist1 val)))))
-	    (when ent
-	      (if (listp ent)
-		  (while ent
-		    (mew-addq ret (car ent))
-		    (setq ent (cdr ent)))
-		(mew-addq ret ent)))
-	    (setq sublist (cdr sublist))))))
-      (setq alist (cdr alist)))
+       ((eq key t)
+	(mew-addq ret alist))
+       ((eq key nil)
+	(or ret (mew-addq ret alist)))
+       ((setq header (mew-header-get-value key))
+	(while alist
+	  (setq val (car (car alist)))
+	  (setq f-or-r (cdr (car alist)))
+	  (setq ent nil)
+	  (when (and (stringp val) (string-match val header))
+	    (cond
+	     ((stringp f-or-r)
+	      (setq ent (mew-refile-guess-by-alist2 val header f-or-r)))
+	     ((listp f-or-r)
+	      (setq ent (mew-refile-guess-by-alist1 f-or-r)))))
+	  (when ent
+	    (if (listp ent)
+		(while ent
+		  (mew-addq ret (car ent))
+		  (setq ent (cdr ent)))
+	      (mew-addq ret ent)))
+	  (setq alist (cdr alist)))))
+      (setq rule (cdr rule)))
     (nreverse ret)))
 
 (defun mew-refile-guess-by-alist2 (regex string template)
