@@ -221,20 +221,20 @@ and retern (beg . end)."
   '("[" mew-summary-buffer-left-msgs " more]" (mew-summary-buffer-raw "*")))
 
 (defun mew-summary-setup-mode-line ()
-  (setq mode-line-format (copy-sequence (default-value 'mode-line-format)))
-  (let* ((tgt mew-mode-line-target)
-	 (target (or (rassoc tgt mode-line-format)
-		     (car (member tgt mode-line-format))))
-	 (prev mode-line-format)
-	 (crnt (cdr prev)))
+  (let ((tgt mew-mode-line-target)
+	target prev pos)
+    (if (boundp 'mode-line-position)
+	(progn
+	  (make-local-variable 'mode-line-position)
+	  (setq mode-line-position
+		(copy-sequence (default-value 'mode-line-position)))
+	  (setq prev mode-line-position))
+      (setq mode-line-format (copy-sequence (default-value 'mode-line-format)))
+      (setq prev mode-line-format))
+    (setq target (or (rassoc tgt prev) (car (member tgt prev))))
     (when target
-      (catch 'loop
-	(while crnt
-	  (when (equal (car crnt) target)
-	    (setcdr prev (append mew-mode-line-format (cdr crnt))) ;; copy
-	    (throw 'loop nil))
-	  (setq prev crnt)
-	  (setq crnt (cdr crnt)))))
+      (setq pos (- (length prev) (length (member target prev))))
+      (setcar (nthcdr pos prev) mew-mode-line-format))
     (or (assq 'mew-summary-buffer-process mode-line-process)
 	(setq mode-line-process
 	      (cons '(mew-summary-buffer-process
