@@ -523,18 +523,24 @@ you can set the sending case."
 ;;; Unlocking
 ;;;
 
-(defun mew-summary-kill-subprocess (&optional no-msg)
+(defun mew-summary-kill-subprocess (&optional kill-ssh)
   "\\<mew-summary-mode-map>
 Kill a process in Summary mode.
 Sometime a process accidentally remains in Summary mode. 
 In this situation, you cannot execute '\\[mew-summary-retrieve]', '\\[mew-summary-ls]', nor '\\[mew-summary-exec]'.
-Use this command to solve this problem."
-  (interactive)
+Use this command to solve this problem.
+
+If called with '\\[universal-argument]', corresponding SSH process, if
+any, is also killed. Please note that the SSH process is shared by 
+other network processes. In this situation, if you kill the SSH process,
+the other network processes are also hung up."
+  (interactive "P")
   (unwind-protect
       (let ((process mew-summary-buffer-process)
-	    pnm)
+	    (disp-msg (interactive-p))
+	    pnm ssh-process)
 	(if (not (processp process))
-	    (or no-msg (message "No process to kill. This buffer is unlocked anyway"))
+	    (if disp-msg (message "No process to kill. This buffer is unlocked anyway"))
 	  (cond
 	   ((eq process t)
 	    ;; Emacs is locking
@@ -542,6 +548,7 @@ Use this command to solve this problem."
 	   ((memq (process-status process) '(open closed))
 	    ;; Network process
 	    (setq pnm (process-name process))
+	    (setq ssh-process (mew-net-get-ssh-process pnm))
 	    (mew-info-clean-up pnm)
 	    (delete-process process)
 	    (mew-summary-visible-buffer (current-buffer))
@@ -549,11 +556,12 @@ Use this command to solve this problem."
 	      (mew-sinfo-set-refile
 	       (nconc (mew-sinfo-get-refile) (mew-sinfo-get-refile-back)))
 	      (mew-sinfo-set-refile-back nil))
+	    (if (and kill-ssh ssh-process) (kill-process ssh-process))
 	    (mew-summary-unlock))
 	   (t
-	    ;; Local process
+	    ;; Local process or SSL
 	    (kill-process process)))
-	  (or no-msg (message "The process was killed"))))
+	  (if disp-msg (message "The process was killed"))))
     (mew-summary-unlock)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
