@@ -1172,12 +1172,16 @@ to this file."
   :group 'mew-draft
   :type 'integer)
 
-
 (defcustom mew-highlight-body-max-size 10000
   "A limit for highlighting a body."
   :group 'mew-highlight
   :type 'integer)
 
+(defcustom mew-scan-max-field-length 5
+  "*Max field length which 'mewls' treats."
+  :group 'mew-env
+  :type 'integer)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Ask?
@@ -1241,6 +1245,11 @@ invalid. Otherwise, 'update is used for range in the case."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-ask-virtual-folder-name nil
+  "*If *non-nil*, ask a virtual folder name."
+  :group 'mew-summary
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Use
